#ifndef __FRACTAL_TYPES_H
#define __FRACTAL_TYPES_H

typedef struct {
} mandel_info;

typedef struct {
    double c_re;
    double c_im;
} julia_info;

typedef enum {
    MANDELBROT, JULIA
} fractal_type;

typedef struct {
    /* coordinates */
    double xmin,xmax,ymax;
    
    /* saved mandelbrot coordinates. we need these when switching back
       from julia mode */
    double old_xmin, old_xmax, old_ymax;
    
    /* recursion depth */
    int depth;

    /* lines done */
    int lines_done;

    /* our idle callback function id */
    gint idle_id;

    /* actual data pointers */
    guint32* raw_data;
    guint32* rgb_data;

    /* TRUE if we're a julia preview */
    gboolean j_pre;
    
    /* it's handy to keep this around */
    GtkWidget* drawing_area;
    
    /* real size. differs from user_size if anti-aliasing is used */
    int real_width;
    int real_height;

    /* user-visible size */
    int user_width;
    int user_height;

    /* anti-aliasing factor. if 1, no anti-aliasing */
    int aa_factor;

    /* fractal type */
    fractal_type fr_type;
    
    /* different fractal types' parameters */
    union {
        mandel_info mandel;
        julia_info julia;
    } u;
} image_info;

void fractal_next_line(image_info* img);

#endif /* __FRACTAL_TYPES_H */
